/*
 * Copyright (c) 2001, Adam Dunkels.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Adam Dunkels.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This file is part of the uIP TCP/IP stack.
 *
 * $Id: uip_arch.c,v 1.1 2001/11/20 20:49:45 adam Exp $
 *
 */


#include "uip_arch.h"
#include "uip.h"

/* This is assembler versions of the uIP support functions for the
   Z80 CPU. Note that the checksum function ("_chksum()", defined
   under uip_ipchksum()) does not support buffer sizes > 255 bytes. */

/*-----------------------------------------------------------------------------------*/
static u16_t chksum_ptr, chksum_len, chksum_tmp;
static u16_t chksum(void);
/*-----------------------------------------------------------------------------------*/
u16_t
uip_ipchksum(void)
{
  chksum_tmp = 0;
  chksum_ptr = (u16_t)&uip_buf[UIP_LLH_LEN];
  chksum_len = 20;
  return chksum();
}
/*asm("_chksum:");*/
u16_t chksum(void) {
_asm
        ld      hl,(_chksum_tmp)
        ld      iy,(_chksum_ptr)
        ld      a,(_chksum_len)
        srl     a
        ld      b,a
        inc     b
        or      a
        jr      00002$
00001$: ;chksum_loop1
        ld      e,(iy)
        inc     iy
        ld      d,(iy)
        inc     iy
        pop     af
        adc     hl,de
00002$:
        push    af
        djnz    00001$ ;chksum_loop1

        ld      a,(_chksum_len)
        srl     a
        jr      nc,00003$ ;chksum_noodd
        ld      e,(iy)
        ld      d,#0
        pop     af
        adc     hl,de
        push    af
00003$: ;chksum_noodd
        ld      de,#0
        pop     af
00004$: ;chksum_endloop:
        adc     hl,de
        jr      c,00004$ ;chksum_endloop
_endasm;
}
/*-----------------------------------------------------------------------------------*/
u16_t
uip_tcpchksum(void)
{
#if 1
  chksum_tmp = 0x0600-0x1401; /* IP_PROTO_TCP */
#else
  chksum_tmp = (u16_t)uip_buf[9 + UIP_LLH_LEN] << 8;
  chksum_tmp += ~0x1400; /* subtract ip header len */
  if (chksum_tmp < ~0x1400) chksum_tmp++;
#endif
  chksum_ptr = (u16_t)&uip_buf[20 + UIP_LLH_LEN];
  chksum_len = 20;
  chksum_tmp = chksum();

#if 0 /* sdcc bug - assignment fails so do it in assembly */
  chksum_ptr = (u16_t) uip_appdata;
#else
_asm
        ld      hl,(_uip_appdata)
        ld      iy,#_chksum_ptr
	ld	0(iy),l
	ld	1(iy),h
_endasm;
#endif
  chksum_len = ((u16_t)uip_buf[2 + UIP_LLH_LEN]<<8) + ((u16_t)uip_buf[3 + UIP_LLH_LEN]&0xff) - 40;
  chksum_tmp = chksum();

  chksum_ptr = (u16_t)&uip_buf[12 + UIP_LLH_LEN];
  chksum_len = 8;
  chksum_tmp = chksum();

  chksum_ptr = (u16_t)&uip_buf[2 + UIP_LLH_LEN];
  chksum_len = 2;
  return chksum();
}
/*-----------------------------------------------------------------------------------*/
#if 0
void
uip_add_rcv_nxt(u8_t n)
{
  uip_conn->rcv_nxt[3] += n;
  if(uip_conn->rcv_nxt[3] < n) {
    uip_conn->rcv_nxt[2]++;
    if(uip_conn->rcv_nxt[2] == 0) {
      uip_conn->rcv_nxt[1]++;
      if(uip_conn->rcv_nxt[1] == 0) {
	uip_conn->rcv_nxt[0]++;
      }
    }
  }
}
#else
void
#if UIP_BUFSIZE > 255
uip_add_rcv_nxt(u16_t n) {
#else
uip_add_rcv_nxt(u8_t n) {
#endif /* UIP_BUFSIZE > 255 */
  /*asm("_uip_add_rcv_nxt:");*/
_asm
        ld      hl,(_uip_conn)
        ld      de,#0x000C
        add     hl,de
        ld      a,(hl)
#if UIP_BUFSIZE > 255
        add     a,5(ix)
        ld      (hl),a
        dec     hl
        ld      a,(hl)
        adc     a,4(ix)
        ld      (hl),a
#else
        add     a,4(ix)
        ld      (hl),a
        jr      nc,00001$
        dec     hl
        inc     (hl)
#endif /* UIP_BUFSIZE > 255 */
        jr      nz,00001$
        dec     hl
        inc     (hl)
        jr      nz,00001$
        dec     hl
        inc     (hl)
00001$:
_endasm;
}
#endif
/*-----------------------------------------------------------------------------------*/
#if 0
void
uip_add_ack_nxt(u8_t n)
{
  uip_conn->ack_nxt[3] += n;
  if(uip_conn->ack_nxt[3] < n) {
    uip_conn->ack_nxt[2]++;
    if(uip_conn->ack_nxt[2] == 0) {
      uip_conn->ack_nxt[1]++;
      if(uip_conn->ack_nxt[1] == 0) {
	uip_conn->ack_nxt[0]++;
      }
    }
  }
}
#else
void
#if UIP_BUFSIZE > 255
uip_add_ack_nxt(u16_t n) {
#else
uip_add_ack_nxt(u8_t n) {
#endif /* UIP_BUFSIZE > 255 */
  /*asm("_uip_add_ack_nxt:");*/
_asm
        ld      hl,(_uip_conn)
        ld      de,#0x0014
        add     hl,de
        ld      a,(hl)
#if UIP_BUFSIZE > 255
        add     a,5(ix)
        ld      (hl),a
        dec     hl
        ld      a,(hl)
        adc     a,4(ix)
        ld      (hl),a
#else
        add     a,4(ix)
        ld      (hl),a
        jr      nc,00001$
        dec     hl
        inc     (hl)
#endif /* UIP_BUFSIZE > 255 */
        jr      nz,00001$
        dec     hl
        inc     (hl)
        jr      nz,00001$
        dec     hl
        inc     (hl)
00001$:
_endasm;
}
#endif
/*-----------------------------------------------------------------------------------*/
